@@ -1,19 +1,14 @@
-from __future__ import print_function
 import numpy as np
 import xarray as xr
 from scipy.signal import filtfilt, gaussian
 from scipy import stats
+
 from dask.array import coarsen, ones_like
 from dask.array.core import Array
 import warnings
 from xarrayutils.utilities import detect_dtype
-
-# from xarrayutils.filtering import filter_1D as filter_1D_refactored
 from xarrayutils.filtering import filter_1D as filter_1D_refactored
 
-# from scipy import optimize
-# import dask.array as dsa
-
 
 """
 Collection of several useful routines for xarray
@@ -36,76 +31,76 @@ def shift_lon(ds, londim, shift=360, crit=0, smaller=True, sort=True):
     return ds
 
 
-def _linregress_ufunc(a, b, nanmask=False):
-    """ufunc to wrap scipy.stats.linregress for xr_linregress"""
-    if nanmask:
-        idxa = np.isnan(a)
-        idxb = np.isnan(b)
-        mask = np.logical_and(~idxa, ~idxb)
-        if sum(~mask) < len(b):  # only applies the mask if not all nan
-            a = a[mask]
-            b = b[mask]
-    slope, intercept, r_value, p_value, std_err = stats.linregress(a, b)
-    return np.array([slope, intercept, r_value, p_value, std_err])
-
-
-def xr_linregress(a, b, dim="time", convert_to_dataset=True, dtype=None, nanmask=False):
-    """Applies scipy.stats.linregress over two xr.DataArrays or xr.Datasets.
+def xr_linregress(x, y, dim="time"):
+    """Calculates linear regression along dimension `dim`.
+    Results are equivalent to `scipy.stats.linregress`.
 
     Parameters
     ----------
-    a : {xr.DataArray}
+    x : {xr.DataArray}
         Independent variable for linear regression. E.g. time.
-    b : {xr.DataArray, xr.Dataset}
+    y : {xr.DataArray, xr.Dataset}
         Dependent variable.
     dim : str
         Dimension over which to perform linear regression.
         Must be present in both `a` and `b` (the default is 'time').
-    convert_to_dataset : bool
-        Converts the output parameter to data_variables instead of an
-        additional dimension (the default is True).
-    dtype : dtype
-         Dtype for the output. If None, defaults to dtype of `b`, or `b`s
-         first data variable(the default is None).
-    nanmask: bool
-        optional masking of nans in the data to avoid nan output from
-        regression (the default is False)
 
     Returns
     -------
     type(b)
-        Returns a dataarray containing the parameter values of
-        scipy.stats.linregress for each data_variable in `b`.
+        Returns a dataarray containing the parameter values
+        for each data_variable in `b`. The naming convention
+        follows `scipy.stats.linregress`
 
     """
+    # align the nan Values before...
+    x = x.where(~np.isnan(y))
+    y = y.where(~np.isnan(x))
+    # TODO: think about making this optional? Right now I err on the side of caution
 
-    if dtype is None:
-        dtype = detect_dtype(b)
+    # Inspired by this post https://stackoverflow.com/a/60352716 but adjusted, so that
+    # results are exactly as with scipy.stats.linregress for 1d vectors.
 
-    stats = xr.apply_ufunc(
-        _linregress_ufunc,
-        a,
-        b,
-        nanmask,
-        input_core_dims=[[dim], [dim], []],
-        output_core_dims=[["parameter"]],
-        vectorize=True,
-        dask="parallelized",
-        output_dtypes=[dtype],
-        output_sizes={"parameter": 5},
+    n = y.notnull().sum(dim)
+
+    nanmask = np.isnan(y).all(dim)
+
+    xmean = x.mean(dim)
+    ymean = y.mean(dim)
+    xstd = x.std(dim)
+    ystd = y.std(dim)
+
+    cov = ((x - xmean) * (y - ymean)).sum(dim) / (n)
+    cor = cov / (xstd * ystd)
+
+    slope = cov / (xstd ** 2)
+    intercept = ymean - xmean * slope
+
+    df = n - 2
+    TINY = 1.0e-20
+    tstats = cor * np.sqrt(df / ((1.0 - cor + TINY) * (1.0 + cor + TINY)))
+    stderr = slope / tstats
+
+    pval = (
+        xr.apply_ufunc(
+            stats.distributions.t.sf,
+            abs(tstats),
+            df,
+            dask="parallelized",
+            output_dtypes=[y.dtype],
+        )
+        * 2
     )
-    stats["parameter"] = xr.DataArray(
-        ["slope", "intercept", "r_value", "p_value", "std_err"], dims=["parameter"]
+
+    return xr.Dataset(
+        {
+            "slope": slope,
+            "intercept": intercept,
+            "r_value": cor.fillna(0).where(~nanmask),
+            "p_value": pval,
+            "std_err": stderr.where(~np.isinf(stderr), 0),
+        }
     )
-    if convert_to_dataset:
-        # chug them all into a dataset
-        ds_stats = xr.Dataset()
-        for ff in stats.parameter.data:
-            ds_stats[ff] = stats.sel(parameter=ff)
-        out = ds_stats
-    else:
-        out = stats
-    return out
 
 
 def linear_trend(obj, dim):
@@ -116,7 +111,7 @@ def linear_trend(obj, dim):
     x = xr.DataArray(
         np.arange(len(obj[dim])).astype(np.float), dims=dim, coords={dim: obj[dim]}
     )
-    trend = xr_linregress(x, obj, dim=dim, convert_to_dataset=False)
+    trend = xr_linregress(x, obj, dim=dim)
     return trend
 
 
@@ -245,7 +240,7 @@ def aggregate(da, blocks, func=np.nanmean, debug=False):
 
 
 def fancymean(raw, dim=None, axis=None, method="arithmetic", weights=None, debug=False):
-    """ extenden mean function for xarray
+    """extenden mean function for xarray
 
     Applies various methods to estimate mean values
     {arithmetic,geometric,harmonic} along specified
@@ -440,18 +435,18 @@ def extractBoxes(da, bo, xname=None, yname=None, xdim="lon", ydim="lat"):
 def dll_dist(dlon, dlat, lon, lat):
     """Converts lat/lon differentials into distances
 
-        PARAMETERS
-        ----------
-        dlon : xarray.DataArray longitude differentials
-        dlat : xarray.DataArray latitude differentials
-        lon  : xarray.DataArray longitude values
-        lat  : xarray.DataArray latitude values
+    PARAMETERS
+    ----------
+    dlon : xarray.DataArray longitude differentials
+    dlat : xarray.DataArray latitude differentials
+    lon  : xarray.DataArray longitude values
+    lat  : xarray.DataArray latitude values
 
-        RETURNS
-        -------
-        dx  : xarray.DataArray distance inferred from dlon
-        dy  : xarray.DataArray distance inferred from dlat
-        """
+    RETURNS
+    -------
+    dx  : xarray.DataArray distance inferred from dlon
+    dy  : xarray.DataArray distance inferred from dlat
+    """
 
     dll_factor = 111000.0
     dx = dlon * xr.ufuncs.cos(xr.ufuncs.deg2rad(lat)) * dll_factor
@@ -701,7 +696,7 @@ def xr_detrend(b, dim="time", trend_params=None, convert_datetime=True):
 
     # Create new time dataarray
     trend_full = t_data * out.slope + out.intercept
-    trend_full = trend_full.assign_coords({dim:b[dim].data})
+    trend_full = trend_full.assign_coords({dim: b[dim].data})
     return b - trend_full
 
 
@@ -734,6 +729,118 @@ def lag_and_combine(ds, lags, dim="time"):
     return xr.concat(datasets, dim=concat_dim_da(lags, "lag"))
 
 
+def sign_agreement(da, ds_ref, dim, threshold=0.75, mask=True):
+    """[summary]
+
+    Parameters
+    ----------
+    da : xr.DataArray
+        Input data
+    ds_ref : xr.DataArray
+        Reference data to compare the sign to . E.g. a mean over `dim`
+    dim : str
+        Dimension of `da` over which the sign agreement is evaluated
+    threshold : float, optional
+        The minimum fraction of elements that have to agree along `dim`, by default 0.75 (75%)
+    mask : bool, optional
+        If True nan values get masked out in the output, by default True
+
+    """
+    if mask:
+        mask_data = np.isnan(da).all(dim)
+    ndim = len(da[dim].data)
+    sign_agreement = (np.sign(da) == np.sign(ds_ref)).sum(dim) >= (threshold * ndim)
+    if mask:
+        sign_agreement = sign_agreement.where(~mask_data)
+    return sign_agreement
+
+
+def mask_mixedlayer(
+    ds,
+    mld,
+    mask="outside",
+    z_dim="lev",
+    z_bounds="lev_bounds",
+    ref_var=None,
+    bound_dim="bnds",
+):
+    """
+    Remove all values from input data `ds` that are above the depth defined by `mld`.
+    If cell bounds are given in the input data, the selection is more accurate, otherwise
+    masking will be perfomed based on cell center values.
+
+    Parameters
+    ----------
+    ds : xr.Dataset
+        Input data
+    mld : xr.Dataarray
+        Mixed Layer Depth input
+    mask : str, optional
+        Switch that determines if values outside (`outside`) or (`inside`) are preserved by the masking
+    z_dim : str, optional
+        Depth dimension of `ds`, by default "lev"
+    z_bounds : str, optional
+        Cell bounds coordinates along `z_dim`, by default "lev_bounds"
+    ref_var : str, optional
+        Reference variable to broadcast against, by default None
+
+    Returns
+    -------
+    xr.Dataset
+        `ds` with mixed layer values replaced by missing values
+    """
+    if ref_var is None:
+        ref_var = list(ds.data_vars)[0]
+
+    # broadcast the mld against a full 3d variable to adjust the chunks...
+    mld = xr.ones_like(ds[ref_var]) * mld
+
+    crit_name = z_dim
+    if z_bounds in ds.coords:
+        crit = ds[z_bounds].isel({bound_dim: 1})
+        crit_name = z_bounds
+        # The proper way to select.
+        # This excludes all cells that have an upper bound bound shallower than the mld
+    else:
+        warnings.warn(
+            "Cell bounds [{z_bounds}] not found in input. Masking is performed with cell centers, which might be less accurate"
+        )
+        crit = ds[z_dim]
+        # Fallback. Use the center depth of the cell.
+        # Could still be influenced by ML, but probably not that bad.
+
+    if mask == "outside":
+        out = ds.where(mld < crit)
+    elif mask == "inside":
+        out = ds.where(mld >= crit)
+    else:
+        raise ValueError("`mask` has to be either `inside` or `outside`")
+    out.attrs.update({"mixed_layer_values_removed_based_on": crit_name})
+    if z_bounds in ds.coords:
+        out = out.assign_coords({z_bounds: ds[z_bounds]})
+    return out
+
+
+def remove_bottom_values(ds, dim="lev", fill_val=-1e10):
+    """Remove the deepest values that are not nan along the dimension `dim`"""
+    # for now assume that values of `dim` increase along the dimension
+    if ds[dim][0] > ds[dim][-1]:
+        raise ValueError(
+            f"It seems like `{dim}` has decreasing values. This is not supported yet. Please sort before."
+        )
+    else:
+        dim_broadcasted = ((ds * 0) + 1) * ds[
+            dim
+        ]  # broadcast and mask missing values according to each variable in the dataset
+        # fill with very large negative value
+        bottom_layer_val = dim_broadcasted.fillna(fill_val).max(dim=dim)
+
+        # now mask, by only retaining the values that are smaller than the bottom value.
+        # this takes also care of columns that have all the same values or are all nan
+        ds_masked = ds.where(dim_broadcasted < bottom_layer_val)
+        return ds_masked
+
+
 ##################
 # Refactored stuff
 
